<?php

namespace Drupal\media_jove\Plugin\media\Source;

use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\Display\EntityFormDisplayInterface;
use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\media\MediaInterface;
use Drupal\media\MediaSourceBase;
use Drupal\media\MediaSourceFieldConstraintsInterface;
use Drupal\media\MediaTypeInterface;
use Drupal\media_jove\JoveUrlResolverInterface;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * JoVE entity media source.
 *
 * @MediaSource(
 *   id = "jove",
 *   label = @Translation("JoVE"),
 *   allowed_field_types = {"string_long"},
 *   default_thumbnail_filename = "jove.png",
 *   description = @Translation("Provides business logic and metadata for JoVE.")
 * )
 */
class Jove extends MediaSourceBase implements MediaSourceFieldConstraintsInterface {

  /**
   * JoVE URL resolver.
   *
   * @var \Drupal\media_jove\JoveUrlResolverInterface
   */
  protected $resolver;

  /**
   * Drupal file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Guzzle HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, EntityFieldManagerInterface $entity_field_manager, FieldTypePluginManagerInterface $field_type_manager, ConfigFactoryInterface $config_factory, JoveUrlResolverInterface $resolver, FileSystemInterface $fileSystem, ClientInterface $httpClient) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $entity_field_manager, $field_type_manager, $config_factory);

    $this->resolver = $resolver;
    $this->fileSystem = $fileSystem;
    $this->httpClient = $httpClient;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.field_type'),
      $container->get('config.factory'),
      $container->get('media_jove.url_resolver'),
      $container->get('file_system'),
      $container->get('http_client')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceFieldConstraints() {
    return [
      'JoveEmbedCode' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadataAttributes() {
    return [
      'id' => $this->t('JoVE Video ID'),
      'title' => $this->t('JoVE Video title'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadata(MediaInterface $media, $attribute_name) {
    $mediaUrl = $this->getSourceFieldValue($media);
    if (!$mediaUrl) {
      return parent::getMetadata($media, $attribute_name);
    }

    switch ($attribute_name) {
      case 'id':
        return $this->resolver->getVideoIdFromEmbedCode($mediaUrl);

      case 'thumbnail_uri':
        return $this->getLocalThumbnailUri($media) ?: parent::getMetadata($media, 'thumbnail_uri');

      case 'title':
        $id = $this->resolver->getVideoIdFromEmbedCode($mediaUrl);
        return $this->getVideoTitle($id) ?: "JoVE video {$media->id()}";

      default:
        break;
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function createSourceField(MediaTypeInterface $type) {
    return parent::createSourceField($type)->set('label', 'JoVE URL');
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceFieldValue(MediaInterface $media) {
    $sourceField = $this->configuration['source_field'];
    if (empty($sourceField)) {
      throw new \RuntimeException('Source field for media source is not defined.');
    }

    /** @var \Drupal\Core\Field\FieldItemInterface $fieldItem */
    $fieldItem = $media->get($sourceField)->first();
    return $fieldItem->{$fieldItem::mainPropertyName()};
  }

  /**
   * {@inheritdoc}
   */
  public function prepareFormDisplay(MediaTypeInterface $type, EntityFormDisplayInterface $display) {
    parent::prepareFormDisplay($type, $display);
    $source_field = $this->getSourceFieldDefinition($type)->getName();

    $display->setComponent($source_field, [
      'type' => 'jove_textfield',
      'weight' => $display->getComponent($source_field)['weight'],
    ]);
    $display->removeComponent('name');
  }

  /**
   * {@inheritdoc}
   */
  public function prepareViewDisplay(MediaTypeInterface $type, EntityViewDisplayInterface $display) {
    $display->setComponent($this->getSourceFieldDefinition($type)->getName(), [
      'type' => 'jove_embed',
    ]);
  }

  /**
   * Returns the local URI for a video thumbnail.
   *
   * If the thumbnail is not already locally stored, this method will attempt
   * to download it.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media entity to get thumbnail URI for.
   *
   * @return string|null
   *   The local thumbnail URI, or NULL if it could not be downloaded.
   */
  protected function getLocalThumbnailUri(MediaInterface $media) {
    $embedCode = $this->getSourceFieldValue($media);
    $id = $this->resolver->getVideoIdFromEmbedCode($embedCode);

    $localPath = file_default_scheme() . '://media-jove/' . HTML::escape($id) . '.jpg';
    if (!file_exists($localPath)) {
      try {
        $dirname = $this->fileSystem->dirname($localPath);
        $this->fileSystem->prepareDirectory($dirname, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);
        system_retrieve_file($this->getOriginalThumbnailUri(), $localPath, FALSE, FileSystemInterface::EXISTS_REPLACE);
      }
      catch (\Exception $e) {
        return NULL;
      }
    }

    return $localPath;
  }

  /**
   * Returns URL to JoVE default thumbnail.
   *
   * @return string
   *   URL to thumbnail image.
   */
  protected function getOriginalThumbnailUri() {
    return 'https://www.jove.com/img/embed_poster.jpg';
  }

  /**
   * Returns title of the video by id.
   *
   * @param int|string $id
   *   Video id.
   *
   * @return string|null
   *   Video title if fetched.
   */
  protected function getVideoTitle($id) {
    $url = $this->resolver->getEmbedUrl($id);

    try {
      $response = $this->httpClient->request('GET', $url->toString());

      if ($response->getStatusCode() == '200') {
        preg_match("/<title>(.*)<\/title>/siU", $response->getBody(), $title_matches);
        list(, $title) = explode(':', preg_replace('/\s+/', ' ', $title_matches[1]));

        return trim($title);
      }

      return NULL;
    }
    catch (\Exception $exception) {
      return NULL;
    }
  }

}
